package net.java.truelicense.core.codec;

import java.nio.charset.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.annotation.CheckForNull;
import net.java.truelicense.obfuscate.Obfuscate;
import org.apache.commons.codec.Charsets;

/**
 * Utilities for codecs.
 *
 * @author Christian Schlichtherle
 */
public class Codecs {

    @Obfuscate static final String APPLICATION_XML_WITH_UTF_8 = "application/xml; charset=utf-8";

    @Obfuscate static final String _7BIT = "7bit";
    @Obfuscate static final String _8BIT = "8bit";
    @Obfuscate static final String QUOTED_PRINTABLE = "quoted-printable";
    @Obfuscate static final String BASE64 = "base64";

    @Obfuscate private static final String CHARSET_PATTERN =
            "\\s*charset\\s*=\\s*(?:\"([^\"]*)\"|([^\\s\\(\\)<>@,;:\\\\\"/\\[\\]\\?=]+))";

    /** Not perfect, but should work for all valid input. */
    private static final Pattern pattern =
            Pattern.compile(CHARSET_PATTERN, Pattern.CASE_INSENSITIVE);

    /**
     * Figures the content transfer charset which is used by the given codec.
     * This method passes the call to {@link #contentTransferCharset}.
     * If an {@link IllegalArgumentException} is thrown, then it returns
     * {@code null}.
     *
     * @param  codec the codec to test.
     * @return The content transfer charset which is used by the given codec or
     *         {@code null} if the codec doesn't produce text or specifies an
     *         invalid charset name or an unknown charset.
     */
    public static @CheckForNull Charset charset(final Codec codec) {
        try { return contentTransferCharset(codec); }
        catch (IllegalArgumentException ex) { return null; }
    }

    /**
     * Figures the content transfer charset which is used by the given codec.
     * <ol>
     * <li>
     * First, the codec's
     * {@linkplain Codec#contentTransferEncoding Content-Transfer-Encoding}
     * gets tested:
     * If it is {@code "7bit"}, {@code "quoted-printable"} or {@code "base64"}
     * (ignoring case), then the returned charset is US-ASCII.
     * <li>
     * Otherwise, if the codec's Content-Transfer-Encoding is {@code "8bit"}
     * (ignoring case), then the codec's {@link Codec#contentType Content-Type}
     * gets tested:
     * <ol>
     * <li>
     * If the codec's Content-Type specifies a {@code charset} parameter
     * (ignoring case), then an attempt is made to load this charset.
     * Upon success, this charset gets returned.
     * <li>
     * If the codec's Content-Type does not specify a {@code charset} parameter,
     * then UTF-8 gets returned.
     * This ensures interoperability with JSON.
     * </ol>
     * <li>
     * Finally, in all other cases, an {@link IllegalArgumentException} gets
     * thrown in order to specify that the codec doesn't produce text or
     * specifies an invalid charset name or an unknown charset.
     * </ol>
     *
     * @param  codec the codec to test.
     * @return The content transfer charset which is used by the given codec.
     * @throws IllegalCharsetNameException if the specified charset name is
     *         invalid.
     * @throws UnsupportedCharsetException if the specified charset is unknown.
     * @throws IllegalArgumentException if the codec doesn't produce text.
     * @since TrueLicense 2.3
     */
    public static @CheckForNull Charset contentTransferCharset(final Codec codec) {
        final String cte = codec.contentTransferEncoding();
        if (    _7BIT.equalsIgnoreCase(cte) ||
                QUOTED_PRINTABLE.equalsIgnoreCase(cte) ||
                BASE64.equalsIgnoreCase(cte)) {
            return Charsets.US_ASCII;
        } else  if (_8BIT.equalsIgnoreCase(cte)) {
            final Matcher m = pattern.matcher(codec.contentType());
            if (m.find()) {
                assert 2 == m.groupCount();
                String n = m.group(1);
                if (null == n) n = m.group(2);
                return Charset.forName(n);
            } else {
                return Charsets.UTF_8;
            }
        }
        throw new IllegalArgumentException(
                "The Content-Transfer-Encoding " + cte + " doesn't produce text.");
    }

    private Codecs() { }
}
